@@ -68,16 +68,17 @@ truefit <- function(x) {
     for ( i in 1:length(nrep) ) {
       log.lik <- llik(x,nrep[i],big)
       if( length( theta[ifree] ) > 0 ){
-        if( i!=length(nrep) ) # no warnings until last optimization
+        if( i!=length(nrep) ) 
           ans <- suppressWarnings( x$options$opt(theta[ifree] , log.lik , low, up) )
         else
-          ans <- x$options$opt( theta[ifree] , log.lik , low, up)
+          ans <- suppressWarnings( x$options$opt( theta[ifree] , log.lik , low, up) )
         theta[ifree] <- ans$estimate
       }
     }
     names(theta) <- names(x$estimate)
     x$estimate <- theta
     x$maximum <- if( length( theta[ifree] ) > 0 ) ans$maximum else -sum( log.lik( theta ) )
+    x$convergence <- ans$convergence
     x
 }
 
@@ -119,9 +120,9 @@ jhess <- function(x, options=x$options, only.jac = FALSE) {
     if (!only.jac) {
         a <- svd(x$jac)
         a$d <- pmax(a$d,sqrt(.Machine$double.eps)*a$d[1])
-        x$hessian <- nlme:::fdHess(rep(0,length(theta.free)),
-                                   function(tx) sum(log.lik(theta.free+a$v%*%(tx/a$d))),
-                                   minAbsPar=1,.relStep=relStep)$Hessian
+        x$hessian <- nlme::fdHess(rep(0,length(theta.free)),
+                                  function(tx) sum(log.lik(theta.free+a$v%*%(tx/a$d))),
+                                  minAbsPar=1,.relStep=relStep)$Hessian
         x$hessian = (x$hessian+t(x$hessian))/2
         x$hessian <- a$v%*%(outer(a$d,a$d)*(x$hessian))%*%t(a$v)
     }
@@ -135,23 +136,40 @@ gradcheck <- function(x, options=x$options) {
     sum(g*solve(crossprod(j),g))
 }
 
-gcmr.opt <- function(start,loglik, lower, upper) {
-    ans <- nlminb(start,function(x) -sum(loglik(x)), lower=lower, upper=upper)
-    if(ans$convergence) warning(paste("nlminb exits with code",ans$convergence))
-    list(estimate=ans$par,maximum=ans$objective)
+gcmr.opt <- function(start, loglik, lower, upper) {
+    fn.opt <- function(x){
+        if( any(x <= lower || x >= upper) ) NA
+        else  -sum(loglik(x))
+    }
+    ans <- optim(start, fn.opt, method="BFGS")
+    if(ans$convergence) warning(paste("optim exits with code",ans$convergence))
+    list(estimate=ans$par,maximum=ans$value,convergence=ans$convergence)
 }
 
 gcmr.options <- function(seed=round(runif(1,1,100000)), nrep=c(100,1000),
-                         no.se=FALSE, opt=gcmr.opt) {
-    list(seed=seed,nrep=nrep,no.se=no.se,opt=opt)
+                         no.se=FALSE, method=c("BFGS", "Nelder-Mead", "CG")) {
+
+    method <- match.arg(method)
+    opt <- function(start, loglik, lower, upper) {
+        fn.opt <- function(x){
+            if( any(x <= lower || x >= upper) ) NA
+            else  -sum(loglik(x))
+        }
+        ans <- optim(start, fn.opt, method=method)
+        if(ans$convergence) warning(paste("optim exits with code",ans$convergence))
+    list(estimate=ans$par,maximum=ans$value,convergence=ans$convergence)
+    }
+    list(seed=seed,nrep=nrep,no.se=no.se,opt=opt,method=method)
 }
 
 gcmr <- function(formula, data, subset, offset, contrasts=NULL, marginal, cormat,
                  start, fixed, options=gcmr.options()){
   call <- match.call()
+  mf <- match.call(expand.dots = FALSE)
   if( !missing(data) ) {
-    marginal <- eval(call$marginal,data,parent.frame())
-    cormat <- eval(call$cormat,data,parent.frame())
+    subset <- eval(mf$subset, data)  
+    marginal <- eval(call$marginal,data,parent.frame()) 
+    cormat <- eval(call$cormat,data[subset,],parent.frame()) 
   }
   if( is.function( marginal ) )
     marginal <- marginal()
@@ -160,9 +178,7 @@ gcmr <- function(formula, data, subset, offset, contrasts=NULL, marginal, cormat
     cormat <- cormat()
   if ( !inherits( cormat , "cormat.gcmr" ) ) stop("Unknown cormat")
   if (missing(data)) 
-    data <- environment(formula)
-  ## next lines partially "inherited" from function glm
-  mf <- match.call(expand.dots = FALSE)
+      data <- environment(formula)
   m <- match(c("formula", "data", "subset", "offset"), names(mf), 0L)
   mf <- mf[c(1L, m)]
   mf$drop.unused.levels <- TRUE
@@ -208,6 +224,7 @@ gcmr <- function(formula, data, subset, offset, contrasts=NULL, marginal, cormat
   if (!is.null(call$offset))
     offsetX <- offsetX + expand_offset(mf[, "(offset)"])
   offset <- list(mean = offsetX, precision = offsetZ)
+  
   fit <- gcmr.fit(X, Y, Z, offset, marginal, cormat, start, fixed, options)
   fit$call <- call
   fit
@@ -238,7 +255,7 @@ gcmr.fit <- function(x=rep(1,NROW(y)), y, z=NULL, offset=NULL, marginal, cormat,
   m <- structure(list(y=as.matrix(y), x=as.matrix(x), z=if(!is.null(z)) as.matrix(z) else NULL,
                       offset=list(mean=as.matrix(offset$mean), precision=as.matrix(offset$precision)),
                       not.na=not.na, n=sum(not.na), marginal=marginal, cormat=cormat, ibeta=1:nb,
-                      igamma=if (ng) (nb+1):(nb+ng) else NULL, call=match.call()), class="gcmr")
+                      igamma=if (ng) (nb+1):(nb+ng) else NULL, nbeta=nb, ngamma=ng, call=match.call()), class="gcmr")
   if ( missing(start) ) {
     lambda <- marginal$start(m$y[not.na,],m$x[not.na,,drop=FALSE],m$z[not.na,,drop=FALSE],
                              list(mean = m$offset$mean[not.na,], precision = m$offset$precision[not.na,]))
@@ -271,9 +288,14 @@ gcmr.fit <- function(x=rep(1,NROW(y)), y, z=NULL, offset=NULL, marginal, cormat,
   ifree <- is.na( m$fixed )
   if( length( m$estimate[ifree] ) == 0 ) ## if all parameters are fixed, skip se
     options$no.se <- TRUE
-  m$options <- do.call(gcmr.options,options)
+  m$options <- options
   ## compute estimate
   m <- truefit(m)
+  m$fitted.values <- marginal$fitted.val(x=m$x[not.na,,drop=FALSE],
+                                         z=m$z[not.na,,drop=FALSE],
+                                         offset=list(mean = m$offset$mean[not.na,],
+                                             precision = m$offset$precision[not.na,]),
+                                         lambda=m$estimate[1:NCOL(x)])
   ## and s.e. and return
   if ( m$options$no.se ) m else jhess(m)
 }
@@ -315,12 +337,11 @@ vcov.gcmr <- function(object, type=c("hessian","sandwich","vscore","cluster","ha
         if (is.null(object$hessian) || is.null(object$jac)) object <- jhess(object)
         if (is.null(object$cormat$id)) stop("no cluster found")
         h <- pinv(object$hessian)
-        v <- sapply(split(1:object$n,object$cormat$id),function(i) colSums(object$jac[i,])) 
+        v <- sapply(split(1:object$n,object$cormat$id[object$not.na]),function(i) colSums(object$jac[i,])) 
         v <- h %*% (v %*% t(v) ) %*% h
     } else {
-        if (!require(sandwich)) stop("HAC requires package sandwich")
         if (is.null(object$hessian) || is.null(object$jac)) object <- jhess(object)
-        v <- vcovHAC(object,...)
+        v <- sandwich::vcovHAC.default(object,...)
     }
     ans <- matrix(0,length(object$estimate),length(object$estimate))
     ifree <- is.na(object$fixed)
@@ -331,27 +352,79 @@ vcov.gcmr <- function(object, type=c("hessian","sandwich","vscore","cluster","ha
 
 se <- function(x,...) sqrt(diag(vcov.gcmr(x,...)))
 
-print.gcmr <- function (x, digits = max(3, getOption("digits") - 3), k = 2 ,...)
+print.gcmr <- function(x, digits = max(3, getOption("digits") - 3), ...)
 {
-    cat("\nCall:", deparse(x$call, width.cutoff = 75), "", sep = "\n")
-    cat("Parameters:\n")
-    par <- x$estimate
-    stderr <- if (x$options$no.se) NULL else se(x,...)
-    xpar <- round( rbind( as.numeric(par) , s.e.=stderr ) , digits=digits )
-    colnames(xpar) <- names(par)
-    print.default(xpar, print.gap = 2)
-    cat("\nlog likelihood = ", format(round(-x$maximum, 2)),
-        ",  aic = ", format(round(AIC(x,k=k), 2)), "\n", sep = "")
+  cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
+
+  if(x$convergence) {
+      cat("model did not converge\n")
+  } else {
+      if( x$nbeta ) {
+          cat("Marginal model parameters:\n")
+          print.default(format(x$estimate[x$ibeta], digits = digits), print.gap = 2, quote = FALSE)
+          cat("\n")
+      } else cat("No parameters in the marginal model\n\n")
+    if( x$ngamma ){
+        cat("Gaussian copula parameters:\n")
+        print.default(format(x$estimate[x$igamma] , digits = digits), print.gap = 2, quote = FALSE)
+        cat("\n")
+    } else cat("No parameters in the Gaussian copula\n\n")
+  }
+  invisible(x)
+}
+
+## summary
+summary.gcmr <- function(object, type = "hessian", ...)
+{
+
+    cf <- object$estimate
+    object$se.type <- match.arg(type, c("hessian", "sandwich", "vscore", "cluster", "hac"))
+    se <- se(object, type = object$se.type, ...)
+    cf <- cbind(cf, se, cf/se, 2 * pnorm(-abs(cf/se)))
+    colnames(cf) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
+    cf <- list(marginal = cf[seq.int(length.out = object$nbeta), , drop = FALSE], 
+               copula = cf[seq.int(length.out = object$ngamma) +  object$nbeta, , drop = FALSE])
+    rownames(cf$marginal) <- names(object$estimate)[seq.int(length.out = object$nbeta)]
+    rownames(cf$copula) <- names(object$estimate)[seq.int(length.out = object$ngamma) + object$nbeta]
+    object$coefficients <- cf
+    object$aic <- AIC(object)
+
+    ## delete some slots
+     object$fitted.values <- object$y <- object$x <- object$z <- object$offset <- object$levels <- object$ibeta <- object$igamma <- object$fixed <- object$hessian <- object$jac <- object$marginal <- object$cormat <- object$not.na <- object$options <- NULL
+
+    class(object) <- "summary.gcmr"
+    object
+}
+
+print.summary.gcmr <- function(x, digits = max(3, getOption("digits") - 3), ...)
+{
+    cat("\nCall:", deparse(x$call, width.cutoff = floor(getOption("width") * 0.85)), "", sep = "\n")
+    if(x$convergence) {
+        cat("model did not converge\n")
+    } else {
+        if( x$nbeta ) {
+            cat("\nCoefficients marginal model:\n")
+            printCoefmat(x$coefficients$marginal, digits = digits, signif.legend = FALSE)
+        } else cat("\nNo coefficients in the marginal model\n")
+
+        if( x$ngamma ) {
+            cat("\nCoefficients Gaussian copula:\n")
+            printCoefmat(x$coefficients$copula, digits = digits, signif.legend = FALSE)
+        } else cat("\nNo coefficients in the Gaussian copula\n")
+        
+        if(getOption("show.signif.stars") & any(do.call("rbind", x$coefficients)[, 4L] < 0.1))
+            cat("---\nSignif. codes: ", "0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1", "\n")
+
+        cat("\nlog likelihood = ", formatC(x$maximum, digits = max(5L, digits + 1L)),
+        ",  AIC = ", format(x$aic, digits = max(4L, digits + 1L)), "\n", sep = "")
+    }
     invisible(x)
 }
 
-## summary/print to be improved in future
-summary.gcmr <- function(object, ...)
-  print.gcmr(object)
 
 ## note: method=interval is hidden
 residuals.gcmr <- function (object, type=c("conditional","marginal"),
-                          method=c("random","mid"),...) {
+                            method=c("random","mid"),...) {
     type <- match.arg(type)
     method <- match.arg(method)
     is.int <- object$marginal$type == "integer"
@@ -404,7 +477,7 @@ plotint <- function(r) {
     for (i in 1:length(tm)) lines(c(tm[i],tm[i]),r[i,])
 }
 
-profile.gcmr <- function(fitted , which , low , up, npoints = 10 , display = TRUE , alpha = 0.05, ... ) {
+profile.gcmr <- function(fitted , which , low , up, npoints = 10 , display = TRUE , alpha = 0.05, progress.bar = TRUE, ... ) {
     if ( !inherits( fitted , "gcmr" ) ) stop("first argument must be a gcmr object")
     if ( which > NCOL( fitted$x ) )
       stop("profile likelihood is computed only for mean response parameters")
@@ -417,11 +490,18 @@ profile.gcmr <- function(fitted , which , low , up, npoints = 10 , display = TRU
       up <- coef(fitted)[which]+3*this.se
     }
     points <- seq( low , up , length = npoints )
+    ## progress bar... 
+    pb <- txtProgressBar(min=0, max=npoints)  
     prof <- function(x,which,fitted) {
         fitted$fixed[which] <- x
-        gcmr:::truefit(fitted)$maximum
+        truefit(fitted)$maximum
     }
-    loglik <- sapply(points,prof,which,fitted)
+    loglik <- rep(NA, npoints)
+    for(i in 1:npoints){
+        loglik[i] <- prof(points[i], which, fitted)
+        if(progress.bar) setTxtProgressBar(pb, i) ## progress bar updated
+    }
+    close(pb)  ## progress bar closed! 
     points <- c( points , fitted$par[which] )
     ord <- order( points)
     points <- points[ord]
@@ -436,6 +516,57 @@ profile.gcmr <- function(fitted , which , low , up, npoints = 10 , display = TRU
     invisible(list(points=points,profile=loglik))
 }
 
+plot.gcmr <- function(x, which = if(!time.series) 1:4 else c(1, 3, 5, 6), caption = c("Residuals vs indices of obs.", "Residuals vs linear predictor", "Normal plot of residuals", "Predicted vs observed values", "Autocorrelation plot of residuals", "Partial ACF plot of residuals"), main = "", ask = prod(par("mfcol")) < length(which) && dev.interactive(), level = 0.95, col.lines = "gray", time.series = FALSE, ...){
+
+    if (!inherits(x, "gcmr")) 
+        stop("use only with \"gcmr\" objects")
+    if (!is.numeric(which) || any(which < 1) || any(which > 7)) 
+        stop("'which' must be in 1:6")
+    res <- residuals(x)
+    fitted.val <- x$fitted.values
+    show <- rep(FALSE, 6)
+    show[which] <- TRUE
+    Main <- rep("", 6)
+    Main[which] <- rep(main, length.out = sum(show))
+    one.fig <- prod(par("mfcol")) == 1
+    if (ask) {
+        op <- par(ask = TRUE)
+        on.exit(par(op))
+    }
+    if (show[1]) {
+        plot(1:length(res), res, xlab = "Obs. number", ylab = "Quantile residual", main = Main[1], 
+            type = if(time.series) "l" else "p", ...)
+        abline(h = 0, lty = 2, col = col.lines, lwd = par()$lwd, ...)
+        mtext(caption[1], 3, 0.25)
+    }
+    if (show[2]) {
+        plot(fitted(x), res, xlab = "Linear predictor", 
+            ylab = "Quantile residual", main = Main[2], ...)
+        abline(h = 0, lty = 2, col = col.lines, lwd = par()$lwd, ...)
+        mtext(caption[2], 3, 0.25)
+    }
+    if (show[3]) {
+        qqPlot(res, envelope = level, grid = FALSE, xlab = "Normal quantiles", ylab = "Sorted quantile residuals", main = Main[3], col.lines = col.lines, lwd = par()$lwd, ...)
+        mtext(caption[3], 3, 0.25)
+    }
+    if (show[4]) {
+        Y <- if(NCOL(x$y)==1) x$y else x$y[,1]/(x$y[,1]+x$y[,2])
+        plot(Y, fitted.val, xlab = "Observed values", ylab = "Predicted values", 
+            main = Main[4], ...)
+        abline(0, 1, lty = 2, col = col.lines, lwd = par()$lwd, ...)
+        mtext(caption[4], 3, 0.25)
+    }
+    if (show[5]) {
+        plot( acf(res, na.action = na.omit, plot = FALSE), ci.col = col.lines, main = Main[5], ...)
+        mtext(caption[5], 3, 0.25)
+    }
+    if (show[6]) {
+        plot( pacf(res, na.action = na.omit, plot = FALSE), ci.col = col.lines, main = Main[6], ...)
+        mtext(caption[6], 3, 0.25)
+    }
+    invisible()
+}
+
 hausman <- function(m, method=c("one-step","full"),
                       nboot=if (method=="one-step") 1000 else 200,
                       nrep=200) {
@@ -513,3 +644,5 @@ dhf <- function(m, mind, sim, nboot) {
     }
     d
 }
+
+
